# -*- coding:utf-8 -*-

# Copyright(C) 2023 Kaytus Inc. All Rights Reserved.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

DOCUMENTATION = '''
---
module: ad_group
version_added: "1.0.0"
author:
    - WangBaoshan (@ieisystem)
short_description: Manage active directory group information
description:
   - Manage active directory group information on kaytus Server.
notes:
   - Does not support C(check_mode).
options:
    state:
        description:
            - Whether the active directory group should exist or not, taking action if the state is different from what is stated.
        choices: ['present', 'absent']
        default: present
        type: str
    name:
        description:
            - Enter the Role Group Name. This name identifies the role group in Active Directory.
        type: str
        required: true
    domain:
        description:
            - Enter the Role Group Domain. This is the domain where the role group is located.
        type: str
    pri:
        description:
            - Enter the Role Group Privilege. This is the level of privilege to be assigned for this role group.
        choices: ['administrator', 'user', 'operator', 'oem', 'none']
        type: str
    kvm:
        description:
            - This field provides access to KVM for AD authenticated role group user.
        choices: ['enable', 'disable']
        type: str
    vm:
        description:
            - This field provides access to VMedia for AD authenticated role group user.
        choices: ['enable', 'disable']
        type: str
extends_documentation_fragment:
    - kaytus.ksmanage.ksmanage
'''

EXAMPLES = '''
- name: Ad group test
  hosts: ksmanage
  connection: local
  gather_facts: false
  vars:
    ksmanage:
      host: "{{ ansible_ssh_host }}"
      username: "{{ username }}"
      password: "{{ password }}"

  tasks:

  - name: "Add active directory group information"
    kaytus.ksmanage.ad_group:
      state: "present"
      name: "wbs"
      domain: "test.com"
      pri: "administrator"
      kvm: "enable"
      vm: "disable"
      provider: "{{ ksmanage }}"

  - name: "Set active directory group information"
    kaytus.ksmanage.ad_group:
      state: "present"
      name: "wbs"
      pri: "user"
      kvm: "disable"
      provider: "{{ ksmanage }}"

  - name: "Delete active directory group information"
    kaytus.ksmanage.ad_group:
      state: "absent"
      name: "wbs"
      provider: "{{ ksmanage }}"
'''

RETURN = '''
message:
    description: Messages returned after module execution.
    returned: always
    type: str
state:
    description: Status after module execution.
    returned: always
    type: str
changed:
    description: Check to see if a change was made on the device.
    returned: always
    type: bool
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.kaytus.ksmanage.plugins.module_utils.ksmanage import (ksmanage_argument_spec, get_connection)


class AD(object):
    def __init__(self, argument_spec):
        self.spec = argument_spec
        self.module = None
        self.init_module()
        self.results = dict()

    def init_module(self):
        """Init module object"""

        self.module = AnsibleModule(
            argument_spec=self.spec, supports_check_mode=False)

    def run_command(self):
        self.module.params['subcommand'] = 'editadgroup'
        self.results = get_connection(self.module)
        if self.results['State'] == 'Success':
            self.results['changed'] = True

    def show_result(self):
        """Show result"""
        self.module.exit_json(**self.results)

    def work(self):
        """Worker"""
        self.run_command()
        self.show_result()


def main():
    argument_spec = dict(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        name=dict(type='str', required=True),
        domain=dict(type='str', required=False),
        pri=dict(type='str', required=False, choices=['administrator', 'user', 'operator', 'oem', 'none']),
        kvm=dict(type='str', required=False, choices=['enable', 'disable']),
        vm=dict(type='str', required=False, choices=['enable', 'disable']),
    )
    argument_spec.update(ksmanage_argument_spec)
    ad_obj = AD(argument_spec)
    ad_obj.work()


if __name__ == '__main__':
    main()
